
/*
  EventManager.cpp
  Copyright (C) 2013  Giménez, Christian N. 
  
  This program is free software: you can redistribute it and/or modify 
  it under the terms of the GNU General Public License as published by 
  the Free Software Foundation, either version 3 of the License, or 
  (at your option) any later version. 
  
  This program is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
  GNU General Public License for more details. 
  
  You should have received a copy of the GNU General Public License 
  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
  
  viernes 11 de enero del 2013    */

#include "EventManager.hpp"
#include <iostream>

using namespace std;

EventManager::EventManager(TinyOgre *tiny, Ogre::RenderWindow* mwin) {    
  
  cout << "EventManager::EventManager()" << endl;

  OIS::ParamList pl;

  this->tiny = tiny;
  size_t window_handler = 0;
  std::ostringstream window_handler_str;
  
  mwin->getCustomAttribute("WINDOW", &window_handler);
  window_handler_str << window_handler;
  pl.insert(std::make_pair(std::string("WINDOW"), window_handler_str.str()));
  
  
  input_manager = OIS::InputManager::createInputSystem( pl );
  keyboard = static_cast<OIS::Keyboard*>(input_manager->createInputObject(OIS::OISKeyboard, true));
  mouse = static_cast<OIS::Mouse*>(input_manager->createInputObject(OIS::OISMouse, true));
  
  
  // Le damos al mouse las coordenadas posibles(width y height).
  unsigned int w,h,d;
  int l,t;
  mwin->getMetrics(w,h,d,l,t);
  
  const OIS::MouseState &ms = mouse->getMouseState();
  ms.width = w;
  ms.height = h;

  cout << "EventManager::EventManager() finished" << endl;
  
}

EventManager::~EventManager(){    
}


void EventManager::registrar(){
  cout << "EventManager::registrar()" << endl;

  key_listener = new MyKeyListener(tiny);
  keyboard->setEventCallback(key_listener);

  tiny->getRoot()->addFrameListener(this);
}

bool EventManager::frameRenderingQueued(const Ogre::FrameEvent& evt){
  // Quitamos esto sino imprimirá a cada segundo el mensaje.
  // cout << "EventManager::frameRenderingQueued()" << endl;
  
  keyboard->capture();
  mouse->capture();

  key_listener->repeatKeyEvent();

  //Ogre::SceneManager* scnMgr = tiny->getSceneMgr();
  //Ogre::Entity* robot = scnMgr->getEntity("Robot");
  //robot->getAnimationState("Shoot")->addTime(evt.timeSinceLastFrame);
  
  
  return true;
}

bool EventManager::frameEnded(const Ogre::FrameEvent& evt){
  return true;
}

bool EventManager::frameStarted(const Ogre::FrameEvent& evt){
  return true;
}
