
    // Game.cpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#include "Game.hpp"
#include "WorldBuilder.hpp"
#include "WorldFile.hpp"

#include "InitFile.hpp"
#include "InitialConfig.hpp"

using namespace Loaders;

#include "Terminal.hpp"

using namespace Terminal;

Game::Game(Ogre::SceneManager *smgr){ 
  Message::show("G", "Initializing game... ***WELCOME***");

  this->smgr = smgr;
  world = NULL;  

  //pc = new PC(smgr->getActiveCamera(), STARTING_PC_MODEL, world->get_current_loc());
  //pc = new PC(STARTING_PC_MODEL, STARTING_PC_TEXTURE, world->get_current_loc());

  Message::show("G", "Loading initial files...");

  InitFile initfile;
  InitialConfig* initconfig = initfile.load();

  this->load_world(initconfig->get_initial_world());

  this->load_location(initconfig->get_initial_location());

  delete(initconfig);    
}

Game::~Game(){
  //delete(pc);
  if (world != NULL){
    delete(world);
  }
}

void Game::load(){
  world->load(this->smgr);
}

bool Game::load_world(string worldname){
  Message::show("G", "Loading world...");
  Message::show("G", worldname.c_str());

  // We erase the current world if worldname is different.
  if ((world != NULL) && (world->get_name() != worldname)){
    delete(world);
  }

  // Load the world.
  WorldBuilder* builder = new WorldBuilder();
  
  WorldFile* file = new WorldFile(builder, worldname);
  
  file->open_read();
  file->load_file();
  world = builder->get_world();

  if (world == NULL){
    Warning::show("G", "The World loaded is NULL!");
  }
  
  file->close();

  delete(builder);
  delete(file);
}

bool Game::load_location(string locationname){
  Message::show("G", "Loading location...");
  Message::show("G", locationname.c_str());
  // Load the location.
  
  world->load_location(locationname);
  
  return true;
}

// void Game::forward(){
//   pc->forward();
// }

// void Game::backward(){
//   pc->backward();
// }

// void Game::left(){
//   pc->left();
// }

// void Game::right(){
//   pc->right();
// }

// void Game::rot_left(){
//   pc->rot_left();
// }

// void Game::rot_right(){
//   pc->rot_right();
// }


// void Game::camera_rot_left(){
//   pc->camera_rot_left();
// }

// void Game::camera_rot_right(){
//   pc->camera_rot_right();
// }

// void Game::camera_nearer(){
//   pc->camera_nearer();
// }

// void Game::camera_farer(){
//   pc->camera_farer();
// }

// void Game::camera_raise(){
//   pc->camera_raise();
// }

// void Game::camera_lower(){
//   pc->camera_lower();
// }

// void Game::play_pc_animation(string animation_name){
//   pc->play_animation(animation_name);
// }

World* Game::get_world(){
  return world;
}

Ogre::SceneManager* Game::get_scenemanager(){
  return smgr;
}

void Game::set_scenemanager(Ogre::SceneManager* smgr_new){
  this->smgr = smgr_new;
}
