
    // Game.hpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#ifndef __GAME_HPP__
#define __GAME_HPP__

#include <OgreRoot.h>
#include <OgreSceneManager.h>

#include "World.hpp"
#include "Game.hpp"
//#include "PC.hpp"

/**
   Abstract representation of the game.
   It load the first things, and let the user play...
 */
class Game {

protected:

  /**
     I need a SceneManager where to load everything needed.
     
     One of the purpose of this is to reduce the amount of parameters and make the interface easier to the developer.
   */
  Ogre::SceneManager *smgr;

  /**
     This is the actual world!
   */
  World *world;

  /**
     This is the main Playable Character(PC).
   */
  //PC *pc;

public:
  /**
     Create a new instance of a Game and load the initial world and location. See InitialConfig and InitFile class documentation.
     
     @param smgr An instance of Ogre::SceneManger where to load the game.
   */
  Game(Ogre::SceneManager *smgr);
  ~Game();

  /**
     @name Loading information.
  */
  //@{
  /**
     Load everything needed for the game again.

     Loads actual world and location.
   */
  void load();

  /**
     I change the actual world by only its name. If the world was loaded accordingly, it is by default loaded the initial location of the world.

     "Changing" means load the file and the world itself into the Ogre::SceneManager.
     
     @param worldname An instance of string with the world's name.
     
     @retval true If everything is loaded accordingly
     @retval false If the world has not been founded or couldn't be loaded.
   */
  bool load_world(string worldname);

  /**
     I change the actual location given only its name. I use the actual world(if there is one loaded!).
     
     @retval true If I could load everything.
     @retval false If I couldn't load the location(maybe it doesn't exists!).

     @params locationname A String with the location's name.
   */
  bool load_location(string locationname);
  //@}

  // void forward();
  // void backward();
  // void left();
  // void right();
  
  // void rot_left();
  // void rot_right();

  // void camera_rot_left();
  // void camera_rot_right();
  // void camera_farer();
  // void camera_nearer();

  // void camera_raise();
  // void camera_lower();

  /**
     Make the player to move according to an animation name.    
     
     @param animation_name is an instance of string that has the name of animation to play.
   */
  //void play_pc_animation(string animation_name);
  
  /**
     @name Getters and Setters
   */
  //@{
  World* get_world();
  
  Ogre::SceneManager* get_scenemanager();
  void set_scenemanager(Ogre::SceneManager *smgr_new);
  //@}
};

#endif 
