
/*
  WorldFile.cpp
  Copyright (C) 2013  Giménez, Christian N. 
  
  This program is free software: you can redistribute it and/or modify 
  it under the terms of the GNU General Public License as published by 
  the Free Software Foundation, either version 3 of the License, or 
  (at your option) any later version. 
  
  This program is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
  GNU General Public License for more details. 
  
  You should have received a copy of the GNU General Public License 
  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
  
  domingo 27 de enero del 2013    */

#include <string>

using namespace std;

#include "WorldFile.hpp"
#include "Terminal.hpp"
#include "common-functions.hpp"

using namespace Terminal;

WorldFile::WorldFile(WorldBuilder* builder, string filename){
  opened = false;

  this->filename =  WF_PATH;
  this->filename = this->filename + filename;
  this->filename = this->filename + "/";
  this->filename = this->filename + filename;

  this->builder = builder;

  next_type = "";
  name = "";
  initial_location = "";
}

WorldFile::~WorldFile(){
}

string WorldFile::next(){
  Message::show("WF", "next()");
  if (!opened){
    // File was not opened!
    Error::show("WF", "File is not opened! Can't continue...");
    next_type = WF_ENDFILE;
    return string(WF_ENDFILE);
  }

  if (file.eof()){
    // Reached end of file!
    close();
    next_type = WF_ENDFILE;
    return string(WF_ENDFILE);
  }

  // Everything is fine... continue :)

  Message::show("WF", "Loading type:");
  
  char buffer[200];
  file.getline(buffer, 200);

  // There is no "trim" function for string, we use the one we created.
  next_type  = CommonFuncs::trim(string(buffer));

  Message::show("WF", next_type.c_str());

  if (next_type.substr(0,5) == "Begin"){
    // take out the "Begin " first characters.
    next_type = next_type.substr(6);
    Message::show("WF", next_type.c_str());
  }
  
  load_element(); //Treat the recently fetched type.

  Message::show("WF", "Finish loading type.");
  return next_type;
}

void WorldFile::load_element(){
  if (next_type == WF_NAME){
    load_name();
    load_initial_location();
    if (builder != NULL){
      builder->build_world(name, initial_location);
    }
  }  

  if (next_type == WF_LOCATIONS){
    if (builder != NULL){
      load_locations_with_builder();
    }else{
      load_locations();
    }
  }
}

void WorldFile::load_name(){
  char buffer[200];
  
  file.getline(buffer,200);
  name = CommonFuncs::trim(string(buffer));

  Message::show("WF", "Loaded world name:");
  Message::show("WF", name.c_str());
}

void WorldFile::load_file(){
  bool cont;
  cont = true;

  if (!opened){
    open_read();
  }

  while (cont){
    this->next();
    
    cont = (next_type != WF_ENDFILE) && (! file.eof()) && opened;
  }

  close();
}

void WorldFile::open_read(){
  if (!opened){
    file.open(filename.c_str(), ios::in);
    opened = true;
    
    Message::show("WF", "File opened:");
    Message::show("WF", filename.c_str());
  }
}

void WorldFile::close(){
  if (opened){
    Message::show("WF", "File closed.");
    file.close();
    opened = false;
  }
}


void WorldFile::load_locations(){
}

void WorldFile::load_locations_with_builder(){
  string location_name;
  char buffer[200];
  
  while (location_name != WF_ENDFILE && !file.eof()){
    // Get the name...
    file.getline(buffer,200);
    location_name = CommonFuncs::trim(string(buffer));
    
    // Send it to the builder...
    if (location_name != WF_ENDFILE){
      builder->add_location(location_name);
      Message::show("WF", "Location name loaded:");
      Message::show("WF", location_name);
    }

    // Keep going!
  }
}

void WorldFile::load_initial_location(){ 
  char buffer[200];

  file.getline(buffer,200);  

  initial_location = CommonFuncs::trim(string(buffer));

  Message::show("WF", "Loaded initial location:");
  Message::show("WF", initial_location.c_str());
}
